
(* :Title: Logic *)

(*:Author: Mark J. Watson, Mathematics Department, Hillsdale College, 
           Hillsdale, MI 49242, March 1990.
*)

(* :Summary:
Definitions for the logical connectives, 'if-then', 'or', 'and', 
'iff', and 'not' are given as functions using 'T'(true) and 'F'(false) as 
arguments and values. Six commands are given which will translate logical 
expressions from Functional, Polish, or Principia into either one of the other 
notations. A command is given which will construct a truth table for user-
defined functions.  Three other commands will test whether an expression is 
tautologous, contradictory, or contingent.
*) 

(* :Context: SetTheory`Logic` *)

(* :Package Version: 1.1 *)

(* :Copyright: *)

(* :History:
Version 1.0 by Mark J. Watson, 1990.
Version 1.1 minor changes made by N. A. Nicorovici, 1999.
*)

(* :Keywords:
elementary logic, logic, logical connectives,  truth-functional 
connectives, Polish notation, Principia notation, propositional calculus, 
truth tables, tautologies, contradictions, contingencies
*)

(* :Source: None. *)

(* :Warning: Changes the usage statements of both 'C' and 'E'. *)

(* :Mathematica Version: 3.0 *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: 
Logic.m is a package for doing problems in propositional calculus.
*)

BeginPackage["SetTheory`Logic`"];

T::usage="The symbol 'T' represents the truth value 'true'.";

F::usage="The symbol 'F' represents the truth value 'false'.";

A::usage="A[p,q] is the disjunction, 'p or q' or 'p v q'.";

C::usage="C[p,q] is the conditional, 'if p then q' or 'p > q'.";

E::usage="E[p,q] is the equivalence, 'p iff q' or 'p = q'.";

K::usage="K[p,q] is the conjunction, 'p and q' or 'p * q'.";

M::usage="M[p] is the negation, 'not p' or '~p'.";

LogicAliases::usage="The command 'LogicAliases' gives convenient aliases for 
the six translation commands using 'Func' for 'Functional', 'Prin' for '
Principia', and 'Pol' for 'Polish'. 'FuncToPol' means 'FunctionalToPolish', '
PrinToPol' means 'PrincipiaToPolish', etc."  

LogicGlossary::usage="The command 'LogicGlossary' gives a glossary of all the 
commands defined in this package.";

PrincipiaGlossary::usage="The command 'PrincipiaGlossary' gives a list of the 
Principia symbols used in this package.";

FunctionalToPolish::usage="FunctionalToPolish[x] converts an expression 'x' 
in Functional notation to an equivalent expression in Polish notation.";

PolishToFunctional::usage="PolishToFunctional[x] converts an expression 'x' 
in Polish notation to an equivalent form in Functional notation.";

FunctionalToPrincipia::usage="FunctionalToPrincipia[x] converts an expression 
'x' in Functional notation to an equivalent form in Principia notation.";

PrincipiaToFunctional::usage="PrincipiaToFunctional[x] converts an expression 
'x' in Principia notation to an equivalent form in Functional notation.";

PrincipiaToPolish::usage="PrincipiaToPolish[x] converts an expression 'x' in 
Principia notation to an equivalent form in Polish notation.";

PolishToPrincipia::usage="PolishToPrincipia[x] converts an expression 'x' in 
Polish notation to an equivalent form in Principia notation.";

TruthTable::usage="TruthTable[n,funcs] is a list of the rows of the truth 
table determined by the functions 'funcs' each defined in the same 'n' 
arguments.  Use TableForm or ColumnForm to convert the list to a table.";  

TautologyQ::usage="TautologyQ[n,func] returns 'True' if the expression 'func' 
defined in 'n' arguments is a tautology, and returns 'False' otherwise.";  

ContradictionQ::usage="ContradictionQ[n,func] returns 'True' if the 
expression 'func' defined in 'n' arguments is a contradiction, and returns '
False' otherwise.";  

ContingentQ::usage="ContingentQ[n,func] returns 'True' if the expression '
func' defined in 'n' arguments is contingent, and returns 'False' otherwise."; 
 
Unprotect[T, F, A, K, M, FunctionalToPolish, PolishToFunctional,
  FunctionalToPrincipia, PrincipiaToFunctional,
  PolishToPrincipia, PrincipiaToPolish, TruthTable,
  TautologyQ, ContradictionQ, ContingentQ,
  PrincipiaGlossary, LogicAliases, LogicGlossary];

Begin["`Private`"];

logicprotected=Unprotect[C, E];

   (*A[p,q] - disjunction*)

A[T,T]=T;A[T,F]=T;A[F,T]=T;A[F,F]=F;

   (*C[p,q] - conditional*)

C[T,T]=T;C[T,F]=F;C[F,T]=T;C[F,F]=T;

   (*E[p,q] - equivalence*)

E[T,T]=T;E[T,F]=F;E[F,T]=F;E[F,F]=T;

   (*K[p,q] - conjunction*)

K[T,T]=T;K[T,F]=F;K[F,T]=F;K[F,F]=F;

   (*M[p] - negation*)

M[T]=F;M[F]=T;

   (*Subroutine for FunctionalToPolish and
     PrincipiaToFunctional*)
     
stringsub[s_,changes_]:=StringJoin@@
  (Characters[ToString[s]]/.changes);

   (*FunctionalToPolish[x]*) 

FunctionalToPolish[x_]:=stringsub[x,{"["->"","]"->"",
  ","->""," "->""}];

   (*PrincipiaToFunctional[x]*) 

PrincipiaToFunctional[x_]:=ToExpression[stringsub[x,
  {"~"->"M@","v"->"~A~",">"->"~C~","="->
  "~E~","*"->"~K~"}]];

   (*PolishToFunctional[x]*)

PolishToFunctional[x_]:=
  First[parse@@Characters[ToString[x]]];

   (*Subroutine for PolishToFunctional*)

parse["M",y__]:=Block[{arg,rest},
  {arg,rest}=parse[y];{M[arg],rest}];

parse[x_,y__]:=Block[{arg1,arg2,rest},
  {arg1,rest}=parse[y];
  {arg2,rest}=parse@@rest;
  {ToExpression[x][arg1,arg2],rest}]/;
  x=="K"||x=="A"||x=="C"||x=="E";
  
parse[x_,y___]:={ToExpression[x],List[y]};
   
   (*FunctionalToPrincipia[x]*) 

FunctionalToPrincipia[M[y_]]:=
  StringJoin["~",FunctionalToPrincipia[y]];

FunctionalToPrincipia[x_[y_,z_]]:=StringJoin["(",
  FunctionalToPrincipia[y],x/.{A->" v ",C->" > ",E->" = ",
  K->" * "},FunctionalToPrincipia[z], ")"];

FunctionalToPrincipia[x_]:=ToString[x];
   
   (*PrincipiaToPolish[x]*) 
 
PrincipiaToPolish[x_]:=
  x//PrincipiaToFunctional//FunctionalToPolish;

   (*PolishToPrincipia[x]*) 

PolishToPrincipia[x_]:=
  x//PolishToFunctional//FunctionalToPrincipia;

   (*Subroutines for TruthTable*)

cases[n_]:=
  Flatten[Outer@@Prepend[Table[{T,F},{n}],List],n-1];
  (*this is the list of all 2^n cases for the truth table*)

row[args_,funcs_]:=Join[args,Apply[#,args]& /@ funcs];
  (*computes a row of the truth table with 'args' placed
    first and then the values of 'funcs' applied to 'args'*)
   
   (*TruthTable[n,funcs]*)

TruthTable[n_Integer?Positive,funcs___]:=Block[{i,header},
  header={"p","q","r","s","t","u","v","w"};

  Prepend[ row[#,{funcs}]& /@ cases[n], 
           Join[Take[header,n],Table[i,{i,n-8}],{funcs}] ] ];

   (*TautologyQ[n,func]*)

TautologyQ[n_Integer?Positive,func_]:=
  FreeQ[Last /@ Rest[TruthTable[n,func]], F];
  
  (*TautologyQ computes TruthTable[n,func], drops the 
    header, and checks for no occurrence of 'F' in the last 
    column*)
  

   (*ContradictionQ[n,func]*)

ContradictionQ[n_Integer?Positive,func_]:=
  FreeQ[Last /@ Rest[TruthTable[n,func]], T]; 
    
  (*ContradictionQ computes TruthTable[n,func], drops the 
    header, and checks for no occurrence of 'T' in the last 
    column*)
  

   (*ContingentQ[n,func]*)

ContingentQ[n_Integer?Positive,func_]:=Block[{l},
  l=(Last /@ Rest[TruthTable[n,func]]);
  MemberQ[l, T] && MemberQ[l, F] ];

  (*ContingentQ[n, func] computes TruthTable[n, func], drops       
    the header, and checks for 'T' and 'F' both occurring in  
    the last column*) 
   
   (*PrincipiaGlossary*)

PrincipiaList={"All symbols are in Courier font.",
"Principia{>, v, *, =, ~} <=> Polish{C, A, K, E, M} <=> Functional{C, A, K,"<>
" E, M} <=> Connective{if-then, or, and, iff, not}",
"Conditional (if-then, >, C):  '>' is the normal 'greater than' sign.",
"Disjunction (or, v, A):  'v' is the normal letter 'v'.",
"Conjunction (and, *, K):  '*' is the normal 'asterisk'.",
"Equivalence (iff, =, E):  '=' is the normal 'equal' sign.",
"Negation (not, ~, M):  '~' is shift '`'."};

PrincipiaGlossary:=TableForm[PrincipiaList];

   (*LogicAliases*)

LogicAliases:=(FuncToPol::=FunctionalToPolish; PolToFunc::=
PolishToFunctional; FuncToPrin::=FunctionalToPrincipia;
PrinToFunc::=PrincipiaToFunctional; PolToPrin::=
PolishToPrincipia; PrinToPol::=PrincipiaToPolish); 

   (*LogicGlossary*) 

LogicList={"A[p,q] is the disjunction, 'p or q' or 'p v q'.",
"C[p,q] is the conditional, 'if p then q' or 'p > q'.",
"ContingentQ[n,func] returns 'True' if the expression 'func' defined in 'n' "<> 
"arguments is contingent, and returns 'False' otherwise.",
"ContradictionQ[n,func] returns 'True' if the expression 'func' defined in"<>
" 'n' arguments is a contradiction, and returns 'False' otherwise.",
"E[p,q] is the equivalence, 'p iff q' or 'p = q'.",
"'F' represents the truth value 'false'.",
"FunctionalToPolish[x] converts 'x' from Functional to Polish notation.",
"FunctionalToPrincipia[x] converts 'x' from Functional to Principia"<> 
" notation.",
"K[p,q] is the conjunction, 'p and q' or 'p * q'.",
"LogicAliases gives aliases for the translation commands using 'Func' for"<>
" 'Functional', 'Prin' for 'Principia', and 'Pol' for 'Polish'. 'FuncToPol' "<>
"means 'FunctionalToPolish', 'PrinToPol' means 'PrincipiaToPolish', etc.",  
"LogicGlossary is a glossary of all commands in the package Logic.m.",
"M[p] is the negation, 'not p' or '~p'.",
"PolishToFunctional[x] converts 'x' from Polish to Functional notation.",
"PolishToPrincipia[x] converts 'x' from Polish to Principia notation.",
"PrincipiaGlossary is a glossary for Principia notation used in Logic.m.",
"PrincipiaToFunctional[x] converts 'x' from Principia to Functional "
"notation.",
"PrincipiaToPolish[x] converts 'x' from Principia to Polish notation.",
"'T' represents the truth value 'True'.",
"TautologyQ[n,func] returns 'True' if the expression 'func' defined in 'n' "<>
"arguments is a tautology, and returns 'False' otherwise.",
"TruthTable[n,funcs] is a list of the rows of the truth table determined by"<>
" the functions 'funcs' each defined in the same 'n' arguments.  Use TableForm"<>
" or ColumnForm to convert the list to a table."};

LogicGlossary:=TableForm[LogicList];

Protect[ Release[logicprotected] ];

End[];

Protect[T, F, A, K, M, FunctionalToPolish, PolishToFunctional,
  FunctionalToPrincipia, PrincipiaToFunctional,
  PolishToPrincipia, PrincipiaToPolish, TruthTable,
  TautologyQ, ContradictionQ, ContingentQ,
  PrincipiaGlossary, LogicAliases, LogicGlossary];

EndPackage[];

