
(* :Title: VennDiagram *)

(* :Author: N. A. Nicorovici, School of Mathematical Sciences, 
            University of Technology, Sydney, NSW 2007, Australia. 
*)

(* :Summary:
The main function of this package is to draw the Venn diagram corresponding 
to a logic expression.
*)

(* :Context: SetTheory`VennDiagram` *)

(* :Package Version: 1.0 *)

(* :Copyright: *)

(* :History:
Version 1.0 by N. A. Nicorovici, 1998. Adapted for MSP, 2001.
*)

(* :Keywords:
	SetTheory, VennDiagram
*)

(* :Source: None. *)

(* :Warning: *)

(* :Mathematica Version: 3.0 or higher *)

(* :Requirements: None *)

(* :Limitations: None known. *)

(* :Discussion: *)

BeginPackage["SetTheory`VennDiagram`", 
            {"SetTheory`ReduceLogical`", "SetTheory`PartsList`"}]

MSPSyntaxCheck::usage =
"MSPSyntaxCheck[expr] returns True if expr is a valid logical expression,
and False otherwise."

StringSyntax::usage =
"StringSyntax[s] is used to catch syntax error messages from MSPSyntaxCheck."

VennDiagram::usage = 
"VennDiagram[expr] draws the Venn diagram for a logical expression 'expr',
VennDiagram[] opens a dialog window."

RandomVennDiagram::usage = 
"RandomVennDiagram[] draws a random Venn diagram and stores its logical
expression."

MatchExpressions::usage = 
"MatchExpressions[expr1, expr2] gives 'True' if the two expressions 'expr1'
and 'expr2' are equivalent, in the sense that they produce the same Venn
diagram, and 'False' otherwise."

ShowParts::usage = 
"ShowParts[expr] draws the intermediary Venn diagrams for the logical 
expression `expr`."

Lc::usage = "Internal function (Protected)."

SyntaxError

stout

outstring

v$protected = Unprotect[MSPSyntaxCheck, StringSyntax, VennDiagram,
                        RandomVennDiagram, MatchExpressions, ShowParts, Lc]

Begin["`Private`"]

venndprotected = Unprotect[draw, fplus, fminus, syntaxcheck]

(* Initialization *)

Green = RGBColor[0.1, 1.0, 0.]

cellbkg = RGBColor[1.,0.380001,0.009995] (* CadmiumOrange *) 

(* Basic Circles *)

a$a = {1, 2, 4, 5}
b$b = {2, 3, 4, 6}
c$c = {4, 5, 6, 7}

d = 2

frame = Line[{{d/2,-d/2},{d/2, d/2}, {-d/2, d/2},
	            {-d/2, -d/2}, {d/2, -d/2}}]

l = d/4; h = l Sqrt[3]/2 ; r = d/5 

{xa, ya} = {-l/2, h/2}
{xb, yb} = { l/2, h/2}
{xc, yc} = { 0, - h/2}

text0 = {
  Text[StyleForm["1", FontSize->18, FontWeight->"Bold"], {xa-d/20, ya+d/25}],
  Text[StyleForm["2", FontSize->18, FontWeight->"Bold"], {0, ya+d/25}],
  Text[StyleForm["3", FontSize->18, FontWeight->"Bold"], {xb+d/20, yb+d/25}],
  Text[StyleForm["4", FontSize->18, FontWeight->"Bold"], {0, ya-d/12}],
  Text[StyleForm["5", FontSize->18, FontWeight->"Bold"], {xa+d/45, ya-d/7}],
  Text[StyleForm["6", FontSize->18, FontWeight->"Bold"], {xb-d/45, yb-d/7}],
  Text[StyleForm["7", FontSize->18, FontWeight->"Bold"], {0, yc-d/20}]
	}

circles = {Circle[{xa, ya}, r], Circle[{xb, yb}, r],
           Circle[{xc, yc}, r]}

text = {
  Text[StyleForm["A", FontSize->16, FontWeight->"Bold"], {xa, ya}],
  Text[StyleForm["B", FontSize->16, FontWeight->"Bold"], {xb, yb}],
  Text[StyleForm["C", FontSize->16, FontWeight->"Bold"], {xc, yc}]
	}

formsetminus = {"\\\\"->"&!", "\\"->"&!", "_"->"&!"}

rules = {"&"->" \[Intersection] ", "|"->" \[Union] ", "!"->" Lc @@ ",
         "A"->"SetTheory`VennDiagram`Private`a$a", 
         "B"->"SetTheory`VennDiagram`Private`b$b", 
         "C"->"SetTheory`VennDiagram`Private`c$c"}

nolf = {"\\012"->"", " "->"", FromCharacterCode[8898]->"&",
                              FromCharacterCode[8899]->"|",
                              FromCharacterCode[172]->"!",
                              FromCharacterCode[63433]->"\\(", 
                              FromCharacterCode[63424]->"\\)"}

outputformat = {"&"->FromCharacterCode[8898],
                "|"->FromCharacterCode[8899],
                "!"->FromCharacterCode[172]}


(* Function Definitions *)

draw[list__,opts___] := Show[Graphics[frame], 
                                       Axes->False,
                                       Epilog->{circles, text}, 
                                       AspectRatio->Automatic,
                                       Prolog->{list},
                                       PlotRange->{{-d/2, d/2}, {-d/2, d/2}},
                                       opts]


fplus[x_, x0_, y0_] := y0 + Sqrt[r^2 - (x - x0)^2]


fminus[x_,x0_,y0_] := y0 - Sqrt[r^2 - (x - x0)^2]
 

Lc[x__] := Complement[Range[1,7], {x}]


MSPSyntaxCheck[s0_String] := Module[
  {s1, nleftb, nrightb, sx, slist, sy, ns, mask, sz, res},
res = False;
s1 = Characters[StringReplace[s0, nolf]];

  If[ Length[s1] == 0,
      Throw[False, SyntaxError[ " No Input"]];
      Return[False]];

 nleftb  = Length[Select[s1,(#=="(")&]];
 nrightb = Length[Select[s1,(#==")")&]];

 sx    = StringJoin[Select[ s1, (!MemberQ[{ " ","!","(",")"}, #])&]];
 slist = {"A"->"1", "B"->"1", "C"->"1", "&"->"0", "|"->"0", "\\\\"->"0",
          "\\"->"0", "_"->"0"};

 sy   = StringReplace[sx, slist];
 ns   = StringLength[sy];
 mask = StringJoin[Map[If[EvenQ[#], "0", "1"]&, Range[ns]]];

 sz   = Length[Select[Characters[sy], (# == "0")&]];

  If[ sz != (ns-1)/2,
      Throw[False, SyntaxError[ " Missing Symbol(s) or Operator(s)"]];
      Return[False]];  

  If[ DigitQ[sy] && sy != mask,
      Throw[False, SyntaxError[ " Missing Symbol(s) or Operator(s)"]]; 
      Return[False]];   

  If[ !DigitQ[sy],
      Throw[False, SyntaxError[ " Wrong Symbol(s) or Operator(s)"]];
      Return[False]];  

  If[ nleftb != nrightb,
      Throw[False, SyntaxError[ " Unbalanced Parentheses"]]; 
      Return[False]];  

res = True
]

MSPSyntaxCheck[s0_]:=False


StringSyntax[e_] := Module[{tmp, res}, 
                    tmp = Catch[MSPSyntaxCheck[e], _SyntaxError, List];
                    res = Which[Head[tmp] === Symbol, Return[{tmp, {}}], 
                                Head[tmp] === List, Return[tmp]]
]


syntaxcheck[s0_String] := Module[
  {s1, nleftb, nrightb, sx, slist, sy, ns, mask, sz, res},
res = False;
s1 = Characters[StringReplace[s0, nolf]];

  If[ Length[s1] == 0,
      NotebookPut[Notebook[{
      Cell["No Input",  
           "Text", FontWeight->"Bold", FontColor->RGBColor[1.,0.,0.],
           FontSize->14, CellFrame->True, Background->None,
           ShowCellBracket->False],
      Cell[ BoxData[ ButtonBox[ RowBox[ {"Close"}],
        ButtonEvaluator->Automatic, 
        ButtonFunction :> NotebookClose[ EvaluationNotebook[ ], 
        Interactive->False], 
            Active -> True]], 
        "Input", ShowCellBracket -> False]
                     }, 
                 WindowSize -> {400, 150}, 
                 WindowMargins -> {{Automatic, 2}, {2, Automatic}}, 
                 WindowElements -> {}, 
                 WindowTitle -> "Error"]];
      Return[False]];

 nleftb  = Length[Select[s1,(#=="(")&]];
 nrightb = Length[Select[s1,(#==")")&]];

 sx    = StringJoin[Select[ s1, (!MemberQ[{ " ","!","(",")"}, #])&]];
 slist = {"A"->"1", "B"->"1", "C"->"1", "&"->"0", "|"->"0", "\\\\"->"0",
          "\\"->"0", "_"->"0"};

 sy   = StringReplace[sx, slist];
 ns   = StringLength[sy];
 mask = StringJoin[Map[If[EvenQ[#], "0", "1"]&, Range[ns]]];

 sz   = Length[Select[Characters[sy], (# == "0")&]];

  If[ sz != (ns-1)/2,
      NotebookPut[Notebook[{
      Cell["Missing Symbol(s) or Operator(s)",  
           "Text", FontWeight->"Bold", FontColor->RGBColor[1.,0.,0.],
           FontSize->14, CellFrame->True, Background->None,
           ShowCellBracket->False],
      Cell[ BoxData[ ButtonBox[ RowBox[ {"Close"}],
        ButtonEvaluator->Automatic, 
        ButtonFunction :> NotebookClose[ EvaluationNotebook[ ], 
        Interactive->False], 
            Active -> True]], 
        "Input", ShowCellBracket -> False]
                     }, 
                 WindowSize -> {400, 150}, 
                 WindowMargins -> {{Automatic, 2}, {2, Automatic}}, 
                 WindowElements -> {}, 
                 WindowTitle -> "Error"]];
     Return[False]];

  If[ DigitQ[sy] && sy != mask,
      NotebookPut[Notebook[{
      Cell["Missing Symbol(s) or Operator(s)",  
           "Text", FontWeight->"Bold", FontColor->RGBColor[1.,0.,0.],
           FontSize->14, CellFrame->True, Background->None,
           ShowCellBracket->False],
      Cell[ BoxData[ ButtonBox[ RowBox[ {"Close"}],
        ButtonEvaluator->Automatic, 
        ButtonFunction :> NotebookClose[ EvaluationNotebook[ ], 
        Interactive->False], 
            Active -> True]], 
        "Input", ShowCellBracket -> False]
                     }, 
                 WindowSize -> {400, 150}, 
                 WindowMargins -> {{Automatic, 2}, {2, Automatic}}, 
                 WindowElements -> {}, 
                 WindowTitle -> "Error"]];
     Return[False]];

  If[!DigitQ[sy],
      NotebookPut[Notebook[{
      Cell["Wrong Symbol(s) or Operator(s): \n"<>
           "You may use only the symbols A, B, C and \n"<>
           "operators & ("<>
           FromCharacterCode[8898]<>"), | ("<>
           FromCharacterCode[8899]<>"), ! ("<>
           FromCharacterCode[172]<>"), \\ ( _ ).",  
           "Text", FontWeight->"Bold", FontColor->RGBColor[1.,0.,0.],
           FontSize->14, CellFrame->True, Background->None,
           ShowCellBracket->False],
      Cell[ BoxData[ ButtonBox[ RowBox[ {"Close"}],
        ButtonEvaluator->Automatic, 
        ButtonFunction :> NotebookClose[ EvaluationNotebook[ ], 
        Interactive->False], 
            Active -> True]], 
        "Input", ShowCellBracket -> False]
                     }, 
                 WindowSize -> {400, 150}, 
                 WindowMargins -> {{Automatic, 2}, {2, Automatic}}, 
                 WindowElements -> {}, 
                 WindowTitle -> "Error"]];
     Return[False]];

  If[ nleftb != nrightb,
      NotebookPut[Notebook[{
      Cell["Unbalanced Parenthesis", 
           "Text", FontWeight->"Bold", FontColor->RGBColor[1.,0.,0.],
           FontSize->14, CellFrame->True, Background->None,
           ShowCellBracket->False],
      Cell[ BoxData[ ButtonBox[ RowBox[ {"Close"}],
        ButtonEvaluator->Automatic, 
        ButtonFunction :> NotebookClose[ EvaluationNotebook[ ], 
        Interactive->False], 
            Active -> True]], 
        "Input", ShowCellBracket -> False]
                     }, 
                 WindowSize -> {400, 150}, 
                 WindowMargins -> {{Automatic, 2}, {2, Automatic}}, 
                 WindowElements -> {}, 
                 WindowTitle -> "Error"]];
     Return[False]];

res = True
]

syntaxcheck[s0_]:=False



(* Nodes *)

eq1 = (x - xa)^2 + (y - ya)^2 - r^2
eq2 = (x - xb)^2 + (y - yb)^2 - r^2
eq3 = (x - xc)^2 + (y - yc)^2 - r^2

(* Intersection of Ca and Cb *)

res12 = Solve[{eq1==0, eq2==0}, {x,y}] // N

{x121, y121} = {x, y} /. res12[[1]]
{x122, y122} = {x, y} /. res12[[2]]

(* Intersection of Cb and Cc *)

res23 = Solve[{eq2==0, eq3==0}, {x,y}] // N

{x231, y231} = {x, y} /. res23[[1]]
{x232, y232} = {x, y} /. res23[[2]]

(* Intersection of Cc and Ca *)

res31 = Solve[{eq3==0, eq1==0}, {x,y}] // N

{x311, y311} = {x, y} /. res31[[1]]
{x312, y312} = {x, y} /. res31[[2]]

(* Roots *)

nodes = {PointSize[0.03],{Point[{x121,y121}],Point[{x122,y122}],
                          Point[{x231,y231}], Point[{x232,y232}],
                          Point[{x311, y311}], Point[{x312,y312}]}}

(* Regions *)

(* Region 1 *)

nl = 100
list1 = Map[{#, fplus[#, xa, ya]}&, 
		Range[xa-r, x122, (x122 - xa + r)/nl]]//N
nl = 50
list2 = Map[{#, fplus[#, xb, yb]}&,Reverse[
		Range[xb-r, x122,(x122 - xb + r)/nl]]] //N
nl = 50
list3 = Map[{#, fminus[#, xb, yb]}&,
		Range[xb-r, x231, (x231 - xb + r)/nl]]//N
nl = 50
list4 = Map[{#, fplus[#, xc, yc]}&,Reverse[
		Range[x311, x231, (x231-x311)/nl]]]//N
nl = 50
list5 = Map[{#, fminus[#, xa, ya]}&,Reverse[
		Range[xa-r, x311, (x311 - xa + r)/nl]]]//N

data1 ={Green,Polygon[Join[Re[list1],Re[list2],Re[list3],Re[list4],
                           Re[list5]]]}

(* Region 2 *)

nl = 50
list1 = Map[{#, fplus[#, xb, yb]}&,Reverse[
		Range[xb-r, x122,(x122 - xb + r)/nl]]] //N
nl = 50
list2 = Map[{#, fminus[#, xb, yb]}&,
		Range[xb-r, x231, (x231 - xb + r)/nl]]//N
nl = 50
list3 = Map[{#, fplus[#, xa, ya]}&,
		Range[x121, xa + r, (xa+r-x121)/nl]]//N
nl = 50
list4 = Map[{#, fminus[#, xa, ya]}&,Reverse[
		Range[x312, xa + r,  (xa + r -x312)/nl]]]//N
nl = 50
list5 = Map[{#, fplus[#, xc, yc]}&, Reverse[
		Range[x231, x312, (x312 - x231)/nl]]]//N

data2 ={Green,Polygon[Join[Re[list1],Re[list2],Re[list3],Re[list4],
                           Re[list5]]]}

(* Region 3 *)

nl = 50
list1 = Map[{#, fplus[#, xa, ya]}&,Reverse[
		Range[x121, xa + r, (xa+r-x121)/nl]]]//N
nl = 100
list2 = Map[{#, fplus[#, xb, yb]}&,
		Range[x121, xb + r, (xb + r - x121)/nl]]//N
nl = 50
list3 = Map[{#, fminus[#, xb, yb]}&, Reverse[
		Range[x232, xb+r, (xb+r-x232)/nl]]]//N
nl = 50
list4 = Map[{#, fplus[#, xc, yc]}&, Reverse[
		Range[x312, x232, (x232 - x312)/nl]]]//N
nl = 50
list5 = Map[{#, fminus[#, xa, ya]}&,
		Range[x312, xa+r, (xa+4-x312)/nl]]//N

data3 ={Green,Polygon[Join[Re[list1],Re[list2], Re[list3], Re[list4],
                           Re[list5]]]}

(* Region 4 *)

nl = 50
list1 = Map[{#, fplus[#, xc, yc]}&, Reverse[
		Range[x231, x312, (x312 - x231)/nl]]]//N
nl = 50
list2 = Map[{#, fminus[#, xb, yb]}&,
		Range[x231, x122, (x122 - x231)/nl]]//N
nl = 50
list3 = Map[{#, fminus[#, xa, ya]}&,
		Range[x122, x312, (x312 - x122)/nl]]//N

data4 ={Green,Polygon[Join[Re[list1],Re[list2], Re[list3]]]}

(* Region 5 *)

nl = 50
list1 = Map[{#, fplus[#, xc, yc]}&,
		Range[x311, x231,  (x231 - x311)/nl]]//N
nl = 50
list2 = Map[{#, fminus[#, xb, yb]}&,
		Range[x231, x122, (x122 - x231)/nl]]//N
nl = 50
list3 = Map[{#, fminus[#, xa, ya]}&, Reverse[
		Range[x311, x122, (x122 - x311)/nl]]]//N

data5 ={Green,Polygon[Join[Re[list1],Re[list2], Re[list3]]]}

(* Region 6 *)

nl = 50
list1 = Map[{#, fplus[#, xc, yc]}&,
		Range[x312, x232,  (x232 - x312)/nl]]//N
nl = 50
list2 = Map[{#, fminus[#, xb, yb]}&,Reverse[
		Range[x122, x232, (x232 - x122)/nl]]]//N
nl = 50
list3 = Map[{#, fminus[#, xa, ya]}&,
		Range[x122, x312, (x312 - x122)/nl]]//N

data6 ={Green,Polygon[Join[Re[list1], Re[list2], Re[list3]]]}

(* Region 7 *)

nl = 150
list1 = Map[{#, fminus[#, xc, yc]}&,
		Range[xc-r, xc + r, 2*r/nl]]//N
nl = 25
list2 = Map[{#, fplus[#, xc, yc]}&, Reverse[
		Range[x232, xc+r, (xc + r - x232)/nl]]]//N
nl = 50
list3 = Map[{#, fminus[#, xb, yb]}&, Reverse[
		Range[x122, x232, (x232 - x122)/nl]]]//N
nl = 50
list4 = Map[{#, fminus[#, xa, ya]}&, Reverse[
		Range[x311, x122, (x122 - x311)/nl]]]//N

nl = 25
list5 = Map[{#, fplus[#, xc, yc]}&, Reverse[
		Range[xc-r, x311, (x311 - xc + r)/nl]]]//N

data7 ={Green,Polygon[Join[Re[list1], Re[list2], Re[list3], Re[list4], 
                           Re[list5]]]}

(* All Regions *)

regs = {data1, data2, data3, data4, data5, data6, data7}

lexp = {"(A & !B & !C)", "(A & B & !C)", "(!A & B & !C)",
        "(A & B & C)", "(A & !B & C)", "(!A & B & C)", "(!A & !B & C)"}



VennDiagram[x_String,opts___] := Module[{s0, s, arg, ind, list, graphics},
                         list = {};
                         If[StringLength[StringReplace[x, nolf]] ==0, 
                                                Return[draw[list,opts]],
                         s0   = StringReplace[x, nolf];
                         list = {};
                         s    = StringReplace[s0, formsetminus]; 
                         arg  = StringReplace[s, rules];
                         ind  = ToExpression[arg];
                         list = Map[regs[[#]]&, ind]];
                         Return[draw[list,opts]]
]

VennDiagram[x_,opts___] := Module[{list},
                   list = {};
                   Return[draw[list,opts]]
]


RandomVennDiagram[x___] := Module[{nr, ni, j, s0, graphics},
                           nr = Random[Integer, {1,7}];
                           ni = Union[Table[Random[Integer, {1,7}], {j, nr}]];
graphics = draw[Map[regs[[#]]&, ni],DisplayFunction->Identity];
s0 = "";
If[Length[ni]==1,
s0 = StringReplace[lexp[[ni[[1]]]], {"("->"", ")"->""}],
s0 = StringReplace[StringJoin @@ Map[lexp[[#]]&, ni], {")("->")|("}]
  ] /; Length[ni] > 0 ;
(*
Print["Initial expression:\n", StringReplace[s0, outputformat]];
*)
stout = s0; (* stout = ReduceExpression[s0]; *)
(*
Print["Simplified expression:\n", stout];
*)
outstring = StringReplace[stout, outputformat];
graphics
]


MatchExpressions[x___] := Module[
  {arg10, arg20, arg1, arg2, s, arg, ind1, ind2, res},

  If[Length[{x}] != 2, CellPrint[Cell["2 arguments expected", "Text", 
                                 FontWeight->"Bold", FontSize->14,
                                 CellFrame->True, Background->cellbkg]];
                       Return[$Failed]];

  {arg10, arg20} = {x};

  If[(!StringQ[arg10])&&(!StringQ[arg20]),
                       CellPrint[Cell["Non-string arguments", 
                         "Text", FontWeight->"Bold", FontSize->14,
                          CellFrame->True, Background->cellbkg]];
                       Return[$Failed]];
  If[!StringQ[arg10],  CellPrint[Cell["The first argument is not a string", 
                         "Text", FontWeight->"Bold", FontSize->14,
                          CellFrame->True, Background->cellbkg]];
                       Return[$Failed]];
  If[!StringQ[arg20],  CellPrint[Cell["The second argument is not a string", 
                         "Text", FontWeight->"Bold", FontSize->14,
                          CellFrame->True, Background->cellbkg]];
                       Return[$Failed]];

  arg1 = StringReplace[arg10, nolf];
  arg2 = StringReplace[arg20, nolf];  
  If[!syntaxcheck[arg1], Return[Null]];
  If[!syntaxcheck[arg2], Return[Null]];

                         s    = StringReplace[arg1, formsetminus]; 
                         arg  = StringReplace[s, rules];
                         ind1 = Union[ToExpression[arg]];

                         s    = StringReplace[arg2, formsetminus]; 
                         arg  = StringReplace[s, rules];
                         ind2 = Union[ToExpression[arg]];

 If[ind1 === ind2, res = "correct", res = "wrong"];

res]

ShowParts[x___] := Module[{stin, stout},
  stin  = StringReplace[x, nolf];
  stout = Map[StringReplace[#, outputformat]&, StringParts[stin]];
  Map[VennDiagram[#,DisplayFunction->Identity,PlotLabel->#]&, stout]
]


SetAttributes[ draw,        ReadProtected]
SetAttributes[ fplus,       ReadProtected]
SetAttributes[ fminus,      ReadProtected]
SetAttributes[ syntaxcheck, ReadProtected]

Protect[Evaluate[venndprotected]]

(* SetAttributes[{a$a, b$b, c$c}, {Protected, Locked}] *)

End[]  (*VennDiagram`Private`*)

SetAttributes[ MSPSyntaxCheck,    ReadProtected]
SetAttributes[ StringSyntax,      ReadProtected]
SetAttributes[ VennDiagram,       ReadProtected]
SetAttributes[ RandomVennDiagram, ReadProtected]
SetAttributes[ MatchExpressions,  ReadProtected]
SetAttributes[ ShowParts,         ReadProtected]
SetAttributes[ Lc,                ReadProtected]

Protect[Evaluate[v$protected]]

EndPackage[] (* VennDiagram`*)


